@@ -15,29 +15,41 @@
 #    under the License.
 """Monasca Agent interface for libvirt metrics"""
 
+import json
+import libvirt
+import math
 import os
 import stat
 import subprocess
 import time
-import yaml
 
 from calendar import timegm
+from copy import deepcopy
 from datetime import datetime
 from distutils.version import LooseVersion
 from monasca_agent.collector.checks import AgentCheck
 from monasca_agent.collector.virt import inspector
+from netaddr import all_matching_cidrs
+
+DOM_STATES = {libvirt.VIR_DOMAIN_BLOCKED: 'VM is blocked',
+              libvirt.VIR_DOMAIN_CRASHED: 'VM has crashed',
+              libvirt.VIR_DOMAIN_NONE: 'VM has no state',
+              libvirt.VIR_DOMAIN_PAUSED: 'VM is paused',
+              libvirt.VIR_DOMAIN_PMSUSPENDED: 'VM is in power management (s3) suspend',
+              libvirt.VIR_DOMAIN_SHUTDOWN: 'VM is shutting down',
+              libvirt.VIR_DOMAIN_SHUTOFF: 'VM has been shut off'}
 
 
 class LibvirtCheck(AgentCheck):
 
     """Inherit Agent class and gather libvirt metrics"""
 
-    def __init__(self, name, init_config, agent_config):
-        AgentCheck.__init__(self, name, init_config, agent_config)
+    def __init__(self, name, init_config, agent_config, instances=None):
+        AgentCheck.__init__(self, name, init_config, agent_config, instances=[{}])
         self.instance_cache_file = "{0}/{1}".format(self.init_config.get('cache_dir'),
-                                                    'libvirt_instances.yaml')
+                                                    'libvirt_instances.json')
         self.metric_cache_file = "{0}/{1}".format(self.init_config.get('cache_dir'),
-                                                  'libvirt_metrics.yaml')
+                                                  'libvirt_metrics.json')
 
     def _test_vm_probation(self, created):
         """Test to see if a VM was created within the probation period.
@@ -48,9 +60,23 @@ class LibvirtCheck(AgentCheck):
         """
         dt = datetime.strptime(created, '%Y-%m-%dT%H:%M:%SZ')
         created_sec = (time.time() - timegm(dt.timetuple()))
-        probation_time = self.init_config.get('vm_probation') - created_sec
+        probation_time = self.init_config.get('vm_probation', 300) - created_sec
         return int(probation_time)
 
+    @staticmethod
+    def _validate_secgroup(cache, instance, source_ip):
+        """Search through an instance's security groups for pingability
+        """
+        for instance_secgroup in instance.security_groups:
+            for secgroup in cache:
+                if ((secgroup['tenant_id'] == instance.tenant_id and
+                     secgroup['name'] == instance_secgroup['name'])):
+                    for rule in secgroup['security_group_rules']:
+                        if ((rule['protocol'] == 'icmp' and
+                             all_matching_cidrs(source_ip,
+                                                [rule['remote_ip_prefix']]))):
+                                return True
+
     def _update_instance_cache(self):
         """Collect instance_id, project_id, and AZ for all instance UUIDs
         """
@@ -58,6 +84,8 @@ class LibvirtCheck(AgentCheck):
 
         id_cache = {}
         flavor_cache = {}
+        port_cache = None
+        netns = None
         # Get a list of all instances from the Nova API
         nova_client = client.Client(2, self.init_config.get('admin_user'),
                                     self.init_config.get('admin_password'),
@@ -68,6 +96,21 @@ class LibvirtCheck(AgentCheck):
                                     region_name=self.init_config.get('region_name'))
         instances = nova_client.servers.list(search_opts={'all_tenants': 1,
                                                           'host': self.hostname})
+        # Lay the groundwork for fetching VM IPs and network namespaces
+        if self.init_config.get('ping_check'):
+            from neutronclient.v2_0 import client
+            nu = client.Client(username=self.init_config.get('admin_user'),
+                               password=self.init_config.get('admin_password'),
+                               tenant_name=self.init_config.get('admin_tenant_name'),
+                               auth_url=self.init_config.get('identity_uri'),
+                               endpoint_type='internalURL')
+            port_cache = nu.list_ports()['ports']
+            # Finding existing network namespaces is an indication that either
+            # DVR agent_mode is enabled, or this is all-in-one (like devstack)
+            netns = subprocess.check_output(['ip', 'netns', 'list'])
+            if netns == '':
+                self.log.warn("Unable to ping VMs, no network namespaces found." +
+                              "Either no VMs are present, or routing is centralized.")
 
         for instance in instances:
             inst_name = instance.__getattr__('OS-EXT-SRV-ATTR:instance_name')
@@ -92,18 +135,57 @@ class LibvirtCheck(AgentCheck):
                         id_cache[inst_name][metadata] = (instance.metadata.
                                                          get(metadata))
 
-            # Try to add private_ip to id_cache[inst_name].  This may fail on ERROR'ed VMs.
-            try:
-                id_cache[inst_name]['private_ip'] = instance.addresses['private'][0]['addr']
-            except KeyError:
-                pass
+            # Build a list of pingable IP addresses attached to this VM and the
+            # appropriate namespace, for use in ping tests
+            if netns:
+                secgroup_cache = nu.list_security_groups()['security_groups']
+                # Find all active fixed IPs for this VM, fetch each subnet_id
+                for net in instance.addresses:
+                    for ip in instance.addresses[net]:
+                        if ip['OS-EXT-IPS:type'] == 'fixed':
+                            subnet_id = None
+                            nsuuid = None
+                            for port in port_cache:
+                                if ((port['mac_address'] == ip['OS-EXT-IPS-MAC:mac_addr']
+                                     and port['tenant_id'] == instance.tenant_id
+                                     and port['status'] == 'ACTIVE')):
+                                    for fixed in port['fixed_ips']:
+                                        if fixed['ip_address'] == ip['addr']:
+                                            subnet_id = fixed['subnet_id']
+                                            break
+                            # Use the subnet_id to find the router
+                            ping_allowed = False
+                            if subnet_id is not None:
+                                for port in port_cache:
+                                    if ((port['device_owner'].startswith('network:router_interface')
+                                         and port['tenant_id'] == instance.tenant_id
+                                         and port['status'] == 'ACTIVE')):
+                                        nsuuid = port['device_id']
+                                        for fixed in port['fixed_ips']:
+                                            if fixed['subnet_id'] == subnet_id:
+                                                # Validate security group
+                                                if self._validate_secgroup(secgroup_cache,
+                                                                           instance,
+                                                                           fixed['ip_address']):
+                                                    ping_allowed = True
+                                                    break
+                                    if nsuuid is not None:
+                                        break
+                            if nsuuid is not None and ping_allowed:
+                                if 'network' not in id_cache[inst_name]:
+                                    id_cache[inst_name]['network'] = []
+                                id_cache[inst_name]['network'].append({'namespace': "qrouter-{0}".format(nsuuid),
+                                                                       'ip': ip['addr']})
+                            elif ping_allowed is False:
+                                self.log.debug("ICMP disallowed for {0} on {1}".format(inst_name,
+                                                                                       ip['addr']))
 
         id_cache['last_update'] = int(time.time())
 
         # Write the updated cache
         try:
-            with open(self.instance_cache_file, 'w') as cache_yaml:
-                yaml.safe_dump(id_cache, cache_yaml)
+            with open(self.instance_cache_file, 'w') as cache_json:
+                json.dump(id_cache, cache_json)
             if stat.S_IMODE(os.stat(self.instance_cache_file).st_mode) != 0o600:
                 os.chmod(self.instance_cache_file, 0o600)
         except IOError as e:
@@ -113,20 +195,19 @@ class LibvirtCheck(AgentCheck):
 
     def _load_instance_cache(self):
         """Load the cache map of instance names to Nova data.
-
            If the cache does not yet exist or is damaged, (re-)build it.
         """
         instance_cache = {}
         try:
-            with open(self.instance_cache_file, 'r') as cache_yaml:
-                instance_cache = yaml.safe_load(cache_yaml)
+            with open(self.instance_cache_file, 'r') as cache_json:
+                instance_cache = json.load(cache_json)
 
                 # Is it time to force a refresh of this data?
                 if self.init_config.get('nova_refresh') is not None:
                     time_diff = time.time() - instance_cache['last_update']
                     if time_diff > self.init_config.get('nova_refresh'):
                         self._update_instance_cache()
-        except (IOError, TypeError):
+        except (IOError, TypeError, ValueError):
             # The file may not exist yet, or is corrupt.  Rebuild it now.
             self.log.warning("Instance cache missing or corrupt, rebuilding.")
             instance_cache = self._update_instance_cache()
@@ -134,34 +215,33 @@ class LibvirtCheck(AgentCheck):
 
         return instance_cache
 
-    def _is_cache_corrupt(self, cache):
-        """Verify that the cache contains a valid dictionary
-        """
-        if cache.__class__.__name__ != 'dict':
-            self.log.warning("Corrupt metrics cache detected.  Will rebuild.")
-            return True
-        return False
-
     def _load_metric_cache(self):
         """Load the counter metrics from the previous collection iteration
         """
         metric_cache = {}
         try:
-            with open(self.metric_cache_file, 'r') as cache_yaml:
-                metric_cache = yaml.safe_load(cache_yaml)
-            if self._is_cache_corrupt(metric_cache):
-                metric_cache = {}
-        except IOError:
+            with open(self.metric_cache_file, 'r') as cache_json:
+                metric_cache = json.load(cache_json)
+        except (IOError, TypeError, ValueError):
             # The file may not exist yet.
+            self.log.warning("Metrics cache missing or corrupt, rebuilding.")
             metric_cache = {}
             pass
 
         return metric_cache
 
-    def _update_metric_cache(self, metric_cache):
+    def _update_metric_cache(self, metric_cache, run_time):
+        # Remove inactive VMs from the metric cache
+        write_metric_cache = deepcopy(metric_cache)
+        for instance in metric_cache:
+            if (('cpu.time' not in metric_cache[instance] or
+                 self._test_vm_probation(time.strftime('%Y-%m-%dT%H:%M:%SZ',
+                                         time.gmtime(metric_cache[instance]['cpu.time']['timestamp'] + run_time))) < 0)):
+                self.log.info("Expiring old/empty {0} from cache".format(instance))
+                del(write_metric_cache[instance])
         try:
-            with open(self.metric_cache_file, 'w') as cache_yaml:
-                yaml.safe_dump(metric_cache, cache_yaml)
+            with open(self.metric_cache_file, 'w') as cache_json:
+                json.dump(write_metric_cache, cache_json)
             if stat.S_IMODE(os.stat(self.metric_cache_file).st_mode) != 0o600:
                 os.chmod(self.metric_cache_file, 0o600)
         except IOError as e:
@@ -290,9 +370,37 @@ class LibvirtCheck(AgentCheck):
                     'timestamp': sample_time,
                     'value': value}
 
+    def _inspect_state(self, insp, inst, instance_cache, dims_customer, dims_operations):
+        """Look at the state of the instance, publish a metric using a
+           user-friendly description in the 'detail' metadata, and return
+           a status code (calibrated to UNIX status codes where 0 is OK)
+           so that remaining metrics can be skipped if the VM is not OK
+        """
+        inst_name = inst.name()
+        dom_status = inst.state()[0] - 1
+        metatag = None
+
+        if inst.state()[0] in DOM_STATES:
+            metatag = {'detail': DOM_STATES[inst.state()[0]]}
+        # A nova-suspended VM has a SHUTOFF Power State, but alternate Status
+        if inst.state() == [libvirt.VIR_DOMAIN_SHUTOFF, 5]:
+            metatag = {'detail': 'VM has been suspended'}
+
+        self.gauge('host_alive_status', dom_status, dimensions=dims_customer,
+                   delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
+                   hostname=instance_cache.get(inst_name)['hostname'],
+                   value_meta=metatag)
+        self.gauge('vm.host_alive_status', dom_status,
+                   dimensions=dims_operations,
+                   value_meta=metatag)
+
+        return dom_status
+
     def check(self, instance):
         """Gather VM metrics for each instance"""
 
+        time_start = time.time()
+
         # Load metric cache
         metric_cache = self._load_metric_cache()
 
@@ -340,18 +448,19 @@ class LibvirtCheck(AgentCheck):
                 self.log.error("{0} is not known to nova after instance cache update -- skipping this ghost VM.".format(inst_name))
                 continue
 
-            # Skip instances that are inactive
-            if inst.isActive() == 0:
-                detail = 'Instance is not active'
-                self.gauge('host_alive_status', 2, dimensions=dims_customer,
-                           delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
-                           hostname=instance_cache.get(inst_name)['hostname'],
-                           value_meta={'detail': detail})
-                self.gauge('vm.host_alive_status', 2, dimensions=dims_operations,
-                           value_meta={'detail': detail})
+            # Accumulate aggregate data
+            for gauge in agg_gauges:
+                if gauge in instance_cache.get(inst_name):
+                    agg_values[gauge] += instance_cache.get(inst_name)[gauge]
+
+            # Skip further processing on VMs that are not in an active state
+            if self._inspect_state(insp, inst, instance_cache,
+                                   dims_customer, dims_operations) != 0:
+                continue
+
+            # Skip the remainder of the checks if alive_only is True in the config
+            if self.init_config.get('alive_only'):
                 continue
-            if inst_name not in metric_cache:
-                metric_cache[inst_name] = {}
 
             # Skip instances created within the probation period
             vm_probation_remaining = self._test_vm_probation(instance_cache.get(inst_name)['created'])
@@ -360,38 +469,8 @@ class LibvirtCheck(AgentCheck):
                                                                                          vm_probation_remaining))
                 continue
 
-            # Test instance's general responsiveness (ping check) if so configured
-            if self.init_config.get('ping_check') and 'private_ip' in instance_cache.get(inst_name):
-                detail = 'Ping check OK'
-                ping_cmd = self.init_config.get('ping_check').split()
-                ping_cmd.append(instance_cache.get(inst_name)['private_ip'])
-                with open(os.devnull, "w") as fnull:
-                    try:
-                        res = subprocess.call(ping_cmd,
-                                              stdout=fnull,
-                                              stderr=fnull)
-                        if res > 0:
-                            detail = 'Host failed ping check'
-                        self.gauge('host_alive_status', res, dimensions=dims_customer,
-                                   delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
-                                   hostname=instance_cache.get(inst_name)['hostname'],
-                                   value_meta={'detail': detail})
-                        self.gauge('vm.host_alive_status', res, dimensions=dims_operations,
-                                   value_meta={'detail': detail})
-                        # Do not attempt to process any more metrics for offline hosts
-                        if res > 0:
-                            continue
-                    except OSError as e:
-                        self.log.warn("OS error running '{0}' returned {1}".format(ping_cmd, e))
-
-            # Skip the remainder of the checks if ping_only is True in the config
-            if self.init_config.get('ping_only'):
-                continue
-
-            # Accumulate aggregate data
-            for gauge in agg_gauges:
-                if gauge in instance_cache.get(inst_name):
-                    agg_values[gauge] += instance_cache.get(inst_name)[gauge]
+            if inst_name not in metric_cache:
+                metric_cache[inst_name] = {}
 
             self._inspect_cpu(insp, inst, instance_cache, metric_cache, dims_customer, dims_operations)
             self._inspect_disks(insp, inst, instance_cache, metric_cache, dims_customer, dims_operations)
@@ -414,9 +493,32 @@ class LibvirtCheck(AgentCheck):
             except KeyError:
                 self.log.debug("Balloon driver not active/available on guest {0} ({1})".format(inst_name,
                                                                                                instance_cache.get(inst_name)['hostname']))
+            # Test instance's remote responsiveness (ping check) if possible
+            if self.init_config.get('ping_check') and 'network' in instance_cache.get(inst_name):
+                for net in instance_cache.get(inst_name)['network']:
+
+                    ping_cmd = self.init_config.get('ping_check').replace('NAMESPACE',
+                                                                          net['namespace']).split()
+                    ping_cmd.append(net['ip'])
+                    dims_customer_ip = dims_customer.copy()
+                    dims_operations_ip = dims_operations.copy()
+                    dims_customer_ip['ip'] = net['ip']
+                    dims_operations_ip['ip'] = net['ip']
+                    with open(os.devnull, "w") as fnull:
+                        try:
+                            self.log.debug("Running ping test: {0}".format(' '.join(ping_cmd)))
+                            res = subprocess.call(ping_cmd,
+                                                  stdout=fnull,
+                                                  stderr=fnull)
+                            self.gauge('ping_status', res, dimensions=dims_customer_ip,
+                                       delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
+                                       hostname=instance_cache.get(inst_name)['hostname'])
+                            self.gauge('vm.ping_status', res, dimensions=dims_operations_ip)
+                        except OSError as e:
+                            self.log.warn("OS error running '{0}' returned {1}".format(ping_cmd, e))
 
         # Save these metrics for the next collector invocation
-        self._update_metric_cache(metric_cache)
+        self._update_metric_cache(metric_cache, math.ceil(time.time() - time_start))
 
         # Publish aggregate metrics
         for gauge in agg_gauges:
