@@ -1,3 +1,5 @@
+# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
+
 import json
 import time
 import urllib2
@@ -62,6 +64,21 @@ ATTRIBUTES = {QUEUE_TYPE: QUEUE_ATTRIBUTES,
               EXCHANGE_TYPE: EXCHANGE_ATTRIBUTES,
               NODE_TYPE: NODE_ATTRIBUTES}
 
+# whitelist of metrics to collect
+DEFAULT_WHITELIST = {
+    QUEUE_TYPE: ['message_stats/deliver_details/rate',
+                 'message_stats/publish_details/rate',
+                 'message_stats/redeliver_details/rate'],
+    EXCHANGE_TYPE: ['message_stats/publish_out',
+                    'message_stats/publish_out_details/rate',
+                    'message_stats/publish_in',
+                    'message_stats/publish_in_details/rate'],
+    NODE_TYPE: ['fd_used',
+                'mem_used',
+                'run_queue',
+                'sockets_used']
+}
+
 DIMENSIONS_MAP = {
     'queues': {'name': 'queue',
                'vhost': 'vhost',
@@ -192,24 +209,15 @@ class RabbitMQ(checks.AgentCheck):
                     self._get_metrics(data_line, object_type, instance)
                     specified_items.remove(name)
 
-        # No queues/node are specified. We will process every queue/node if it's under the limit
+        # No queues/node are specified. We will process every queue/node
         else:
-            # Monasca does not support events at this time.
-            # if len(data) > ALERT_THRESHOLD * max_detailed:
-            #   # Post a message on the dogweb stream to warn
-            #   self.alert(base_url, max_detailed, len(data), object_type)
-
-            if len(data) > max_detailed:
-                # Display a warning in the info page
-                self.warning(
-                    "Too many queues to fetch. You must choose the %s you are interested in by editing the rabbitmq.yaml configuration file" %
-                    object_type)
-
-            for data_line in data[:max_detailed]:
-                # We truncate the list of nodes/queues if it's above the limit
+            for data_line in data:
                 self._get_metrics(data_line, object_type, instance)
 
     def _get_metrics(self, data, object_type, instance):
+        whitelist = instance.get('whitelist', {})
+        object_whitelist = whitelist.get(object_type, DEFAULT_WHITELIST[object_type])
+
         dimensions_list = DIMENSIONS_MAP[object_type].copy()
         dimensions = self._set_dimensions({'component': 'rabbitmq', 'service': 'rabbitmq'},
                                           instance)
@@ -219,6 +227,10 @@ class RabbitMQ(checks.AgentCheck):
                 dimensions[dimensions_list[d]] = dim
 
         for attribute, metric_name in ATTRIBUTES[object_type]:
+            # Check if the metric should be collected
+            if attribute not in object_whitelist:
+                continue
+
             # Walk down through the data path, e.g. foo/bar => d['foo']['bar']
             root = data
             keys = attribute.split('/')
@@ -232,7 +244,7 @@ class RabbitMQ(checks.AgentCheck):
                 self.log.debug("Collected data for %s: metric name: %s: value: %f dimensions: %s" % (object_type, metric_name, float(value), str(dimensions)))
                 self.gauge('rabbitmq.%s.%s' % (METRIC_SUFFIX[object_type], metric_name), float(value), dimensions=dimensions)
             except ValueError:
-                self.log.debug("Caught ValueError for %s %s = %s  with dimensions: %s" % (
+                self.log.exception("Caught ValueError for %s %s = %s  with dimensions: %s" % (
                     METRIC_SUFFIX[object_type], attribute, value, dimensions))
 
     def alert(self, base_url, max_detailed, size, object_type):
