@@ -1,3 +1,5 @@
+# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
+
 """Classes for detection of running resources to be monitored.
 
     Detection classes should be platform independent
@@ -48,6 +50,8 @@ class Plugin(object):
         :return: An agent_config.Plugins object
         """
         conf = self.build_config()
+        if conf is None:
+            return None
         for plugin_type in conf.itervalues():
             for inst in plugin_type['instances']:
                 inst['built_by'] = self.__class__.__name__
