@@ -1,3 +1,5 @@
+# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
+
 import monasca_setup.detection
 
 
@@ -13,7 +15,9 @@ class Keystone(monasca_setup.detection.ServicePlugin):
             'template_dir': template_dir,
             'overwrite': overwrite,
             'service_name': 'identity-service',
-            'process_names': ['keystone-admin', 'keystone-main'],
+            'process_names': ['keystone-admin',
+                              'keystone-main',
+                              'keystone-all'],
             'service_api_url': 'http://localhost:35357/v3',
             'search_pattern': '.*v3\..*'
         }
