@@ -1,3 +1,5 @@
+# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
+
 import logging
 
 import monasca_setup.agent_config
@@ -33,7 +35,7 @@ class MySQL(monasca_setup.detection.Plugin):
         """
         config = monasca_setup.agent_config.Plugins()
         # First watch the process
-        config.merge(monasca_setup.detection.watch_process(['mysqld']))
+        config.merge(monasca_setup.detection.watch_process(['mysqld'], 'mysql'))
         log.info("\tWatching the mysqld process.")
 
         # Attempt login, requires either an empty root password from localhost
